package org.eclipse.cosmos.rm.smlif.internal.export;

import java.io.File;
import java.io.IOException;
import java.util.Map;

import junit.framework.Test;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Path;
import org.eclipse.cosmos.rm.smlif.internal.common.TestUtilities;
import org.eclipse.cosmos.rm.validation.internal.core.IValidator;
import org.eclipse.cosmos.rm.validation.internal.core.MainValidator;
import org.eclipse.hyades.test.common.junit.DefaultTestArbiter;
import org.eclipse.hyades.test.common.junit.HyadesTestCase;
import org.eclipse.hyades.test.common.junit.HyadesTestSuite;

/**
 * Generated code for the test suite <b>TestExportToSMLIF</b> located at
 * <i>/org.eclipse.cosmos.rm.smlif.tests/plugin-junits/TestExportToSMLIF.testsuite</i>.
 *
 * test ExportToSMLIF
 */
public class TestExportToSMLIF extends HyadesTestCase {
	/**
	 * Constructor for TestExportToSMLIF.
	 * @param name
	 */
	public TestExportToSMLIF(String name) {
		super(name);
	}

	/**
	 * Returns the JUnit test suite that implements the <b>TestExportToSMLIF</b> definition.
	 */
	public static Test suite() {
		HyadesTestSuite testExportToSMLIF = new HyadesTestSuite("TestExportToSMLIF");
		testExportToSMLIF.setArbiter(DefaultTestArbiter.INSTANCE).setId("E11BD6C63466E73F6BB191D0A65211DB");
		testExportToSMLIF.addTest(new TestExportToSMLIF("testExport").setId("E11BD6C63466E73F7EAE41C0A65211DB").setTestInvocationId("E11BD6C63466E73F8A8CE460A65211DB"));
		return testExportToSMLIF;
	}

	/**
	 * testExport
	 * @throws Exception
	 */
	public void testExport() throws Exception {
		performExports(new String[]{
				"/TestSMLSample/acyclic/Other",
//				"/TestSMLSample/acyclic/ValidCycles1/course1.xml",
//				"/TestSMLSample/acyclic/ValidCycles2/course1.xml",				
//				"/TestSMLSample/acyclic/Other/course1",
				}, false);

	}

	private void performExports(String[] directoryNames, boolean shouldThrowException) {
		for (int i = 0; i < directoryNames.length; i++) {
			File tempFile = null;
			try {
				tempFile = File.createTempFile("foo", ".xml");
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			String dirName = directoryNames[i];
			IFile iFile = ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(dirName));
			ExportToSMLIF importer = new ExportToSMLIF();
			boolean threwException = false;
//			try {
//				// TODO need to fix for new API
////				importer.buildSMLIFFileFromFolder(iFile.getLocation().toOSString(), tempFile.getAbsolutePath());
//			} catch (ExportException e) {
//				threwException = true;
//			} finally {
//			}
			
			String not = shouldThrowException ? "" : "not ";
			assertEquals("Exporting directory "+dirName+" should "+not+"throw an export exception", shouldThrowException, threwException);
			
			assertEquals("SML-IF file should "+not+"be empty", shouldThrowException, TestUtilities.isEmpty(tempFile));
			
			if (!shouldThrowException) {
				// TODO call validator on new SML-IF file and assert that it's valid
				assertTrue("SML-IF is valid", validate(tempFile));
			}
			tempFile.delete();
		}
	}

	private boolean validate(File tempFile) {
		MainValidator validator = new MainValidator();
		Map attributes = validator.getAttributes();
		attributes.put(IValidator.ATTRIBUTE_INPUT_TYPE, IValidator.VALUE_SML_IF);
		attributes.put(IValidator.ATTRIBUTE_INSTANCE, tempFile.getAbsolutePath());						
		attributes.put(IValidator.ATTRIBUTE_ENV, IValidator.VALUE_ENV_ECLIPSE);		
		return validator.start();		
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		//tempDir = new File(System.getProperty("java.io.tmpdir")+File.separatorChar+"foo");
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		//TestUtilities.deleteDirectory(tempDir);
	}

}
