package org.eclipse.cosmos.rm.smlif.internal.importing;

import java.io.File;

import junit.framework.Test;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Path;
import org.eclipse.cosmos.rm.smlif.internal.common.TestUtilities;
import org.eclipse.hyades.test.common.junit.DefaultTestArbiter;
import org.eclipse.hyades.test.common.junit.HyadesTestCase;
import org.eclipse.hyades.test.common.junit.HyadesTestSuite;

/**
 * Generated code for the test suite <b>TestImportFromSMLIF</b> located at
 * <i>/org.eclipse.cosmos.rm.smlif.tests/plugin-junits/TestImportFromSMLIF.testsuite</i>.
 *
 * test ImportFromSMLIF
 */
public class TestImportFromSMLIF extends HyadesTestCase {
	private File tempDir;

	/**
	 * Constructor for TestImportFromSMLIF.
	 * @param name
	 */
	public TestImportFromSMLIF(String name) {
		super(name);
	}

	/**
	 * Returns the JUnit test suite that implements the <b>TestImportFromSMLIF</b> definition.
	 */
	public static Test suite() {
		HyadesTestSuite testImportFromSMLIF = new HyadesTestSuite("TestImportFromSMLIF");
		testImportFromSMLIF.setArbiter(DefaultTestArbiter.INSTANCE).setId("CB73A2710F12A7CF837B89A0A64411DB");
		testImportFromSMLIF.addTest(new TestImportFromSMLIF("testImport").setId("CB73A2710F12A7CF94F32030A64411DB").setTestInvocationId("CB73A2710F12A7CF9F974700A64411DB"));
		return testImportFromSMLIF;
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		tempDir = new File(System.getProperty("java.io.tmpdir")+File.separatorChar+"foo");
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		TestUtilities.deleteDirectory(tempDir);
	}

	/**
	 * testImport
	 * @throws Exception
	 */
	public void testImport() throws Exception {
		performImports(new String[]{
				"/TestSMLSample/targetType/Valid.xml",
//				"/TestSMLSample/acyclic/ValidCycles1/course1.xml",
//				"/TestSMLSample/acyclic/ValidCycles2/course1.xml",				
//				"/TestSMLSample/acyclic/Other/course1",
				}, false, new int[] {5});

		performImports(new String[]{
				"/TestSMLSample/Incomplete/InValid.xml",
//				"/TestSMLSample/acyclic/ValidCycles1/course1.xml",
//				"/TestSMLSample/acyclic/ValidCycles2/course1.xml",				
//				"/TestSMLSample/acyclic/Other/course1",
				}, true, new int[] {0});
	}

	
	private void performImports(String[] files, boolean shouldThrowException, int[] numberOfFilesCreated) {
		for (int i = 0; i < files.length; i++) {
			TestUtilities.deleteDirectory(tempDir);
			String fileName = files[i];
			IFile iFile = ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(fileName));
			ImportFromSMLIF importer = new ImportFromSMLIF();
			tempDir.mkdir();
			boolean threwException = false;
			try {
				importer.importFromFileSystemFile(iFile.getLocation().toOSString(), tempDir.getAbsolutePath());
			} catch (ImportException e) {
				threwException = true;
			} finally {
			}
			
			String not = shouldThrowException ? "" : "not ";
			assertTrue("Importing file "+fileName+" should "+not+"throw an import exception", shouldThrowException == threwException);
			
			assertEquals("Target directory file count test", numberOfFilesCreated[i], tempDir.list().length);
		}
	}

}
