/**********************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.smlif.internal.editor;

import org.eclipse.cosmos.rm.smlif.internal.SMLPlugin;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLCommonUtil;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLMessages;
import org.eclipse.cosmos.rm.validation.internal.common.IValidationConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.widgets.ScrolledForm;

/**
 * The SML-IF page is used to display the entire content of the
 * SML-IF document in XML format
 * 
 * @author Ali Mehregani
 */
public class SMLIFPage extends AbstractFormPage
{
	/**
	 * The id of this page
	 */
	public static final String ID = SMLPlugin.PLUGIN_ID + ".SMLIFPage";
	
	/**
	 * Caches the content of the SML-IF document
	 */
	private StringBuffer buffer;

	/**
	 * The styled text used to display the content of the
	 * document
	 */
	private StyledText xmlContent;
	
	/**
	 * Constructor
	 * 
	 * @param editor The form editor
	 */
	public SMLIFPage(FormEditor editor)
	{
		super(editor, ID, SMLMessages.editorSMLIFTitle);
		buffer = new StringBuffer();
	}
	
	protected void createFormContent(IManagedForm managedForm)
	{
		managedForm.addPart(((SMLIFEditor)getEditor()).getSmlifFormPart());
		ScrolledForm form = managedForm.getForm();
		form.setText(getTitle());
		
		Composite parent = form.getBody();	
		GridLayout gridLayout = new GridLayout();
		gridLayout.marginTop = 10;	
		parent.setLayout(gridLayout);
		parent.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		SMLIFEditor editor = (SMLIFEditor)getEditor();
		xmlContent = editor.getExtendedToolkit().createStyledtext(parent, IValidationConstants.EMPTY_STRING, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
		xmlContent.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		xmlContent.addLineStyleListener(new XMLLineStyler());
		xmlContent.setFont(new Font(Display.getDefault(), "Courier New", 10, SWT.NORMAL)); //$NON-NLS-1$
		xmlContent.setEditable(false);
		
		form.setEnabled(isEnabled());
	}
	
	
	public void setActive(boolean active)
	{	
		super.setActive(active);		
		if (!active)
			return;
		
		SMLIFEditor editor = (SMLIFEditor)getEditor();
		if (buffer.length() <= 0 || editor.isDirty())
		{
			buffer = new StringBuffer();
			SMLCommonUtil.serialzeNode(buffer, editor.getDocumentNode(), 0);
		}
				
		xmlContent.setText(buffer.toString());
	}
	
	public void dispose()
	{		
		super.dispose();
		if (xmlContent != null)
		{
			xmlContent.dispose();
			xmlContent = null;
		}
		
		buffer = null;
	}

	protected void createLeftColumn(Composite parent)
	{
		/* Doesn't need to be implemented */		
	}

	protected void createRightColumn(Composite parent)
	{
		/* Doesn't need to be implemented */
	}
}
