package org.eclipse.cosmos.rm.validation.internal.core;

import java.io.IOException;
import java.util.Map;

import junit.framework.Test;

import org.eclipse.cosmos.rm.validation.internal.common.FileOutput;
import org.eclipse.cosmos.rm.validation.internal.common.UtilityTester;
import org.eclipse.hyades.test.common.junit.DefaultTestArbiter;
import org.eclipse.hyades.test.common.junit.HyadesTestCase;
import org.eclipse.hyades.test.common.junit.HyadesTestSuite;

/**
 * Generated code for the test suite <b>TestPluginMainValidator</b> located at
 * <i>/org.eclipse.cosmos.rm.validation.tests/plugin-junits/TestPluginMainValidator.testsuite</i>.
 */
public class TestPluginMainValidator extends HyadesTestCase
{
	/**
	 * Constructor for TestPluginMainValidator.
	 * @param name
	 */
	public TestPluginMainValidator(String name)
	{
		super(name);
	}

	/**
	 * Returns the JUnit test suite that implements the <b>TestPluginMainValidator</b> definition.
	 */
	public static Test suite()
	{
		HyadesTestSuite testPluginMainValidator = new HyadesTestSuite("TestPluginMainValidator");
		testPluginMainValidator.setArbiter(DefaultTestArbiter.INSTANCE).setId("C205F1E79253A3D1872346609CF811DB");
		testPluginMainValidator.addTest(new TestPluginMainValidator("testDataCenterAbstract").setId("C205F1E79253A3D1E56CAEF09F6511DB").setTestInvocationId("C205F1E79253A3D1EF7950A09F6511DB"));
		testPluginMainValidator.addTest(new TestPluginMainValidator("testDataCenterHardware").setId("C205F1E79253A3D152B385309F6D11DB").setTestInvocationId("C205F1E79253A3D12B5151109F6E11DB"));
		testPluginMainValidator.addTest(new TestPluginMainValidator("testDataCenterOS").setId("C205F1E79253A3D152FB15809F6D11DB").setTestInvocationId("C205F1E79253A3D12B587D009F6E11DB"));
		testPluginMainValidator.addTest(new TestPluginMainValidator("testDataCenterSuperTypes").setId("C205F1E79253A3D155CD6F609F6D11DB").setTestInvocationId("C205F1E79253A3D12B5AC6F09F6E11DB"));
		testPluginMainValidator.addTest(new TestPluginMainValidator("testDataCenterSoftware").setId("C205F1E79253A3D158E759909F6D11DB").setTestInvocationId("C205F1E79253A3D12B5F81E09F6E11DB"));
		testPluginMainValidator.addTest(new TestPluginMainValidator("testDataCenterAll").setId("C205F1E79253A3D148B08120A00111DB").setTestInvocationId("C205F1E79253A3D14EF4F750A00111DB"));
		testPluginMainValidator.addTest(new TestPluginMainValidator("testAcyclicInvalidCycles").setId("C205F1E79253A3D1966BE100A00311DB").setTestInvocationId("C205F1E79253A3D19EE0A4B0A00311DB"));
		testPluginMainValidator.addTest(new TestPluginMainValidator("testAcyclicInvalidDerivation").setId("C205F1E79253A3D183866290A00311DB").setTestInvocationId("C205F1E79253A3D1C40E5020A00311DB"));
		testPluginMainValidator.addTest(new TestPluginMainValidator("testAcyclicValidCycles1").setId("C205F1E79253A3D1862B6AE0A00311DB").setTestInvocationId("C205F1E79253A3D1C4133220A00311DB"));
		testPluginMainValidator.addTest(new TestPluginMainValidator("testAcyclicValidCycles2").setId("C205F1E79253A3D1868AC8F0A00311DB").setTestInvocationId("C205F1E79253A3D1C4157C10A00311DB"));
		testPluginMainValidator.addTest(new TestPluginMainValidator("testValidDerivation").setId("C205F1E79253A3D186F152F0A00311DB").setTestInvocationId("C205F1E79253A3D1C41A3700A00311DB"));
		return testPluginMainValidator;
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception
	{
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception
	{
	}

	private void validateModelUnits(String[] units, String output, String expectedOutput, boolean pass) throws IOException
	{
		MainValidator mainValidator = new MainValidator();
		Map attributes = mainValidator.getAttributes();
		
		String unitFiles = "";
		for (int i = 0; i < units.length; i++)
		{
			if (i > 0)
				unitFiles += ",";
			unitFiles += units[i];
		}
		
		attributes.put(IValidator.ATTRIBUTE_INPUT_TYPE, IValidator.VALUE_SML_UNIT);
		attributes.put(IValidator.ATTRIBUTE_INSTANCE, unitFiles);
		attributes.put(IValidator.ATTRIBUTE_OUTPUT, new FileOutput());
		attributes.put(IValidator.ATTRIBUTE_FILE_OUTPUT, output);	
		boolean validation = mainValidator.start();		
		UtilityTester.compareFiles(expectedOutput, output);		
		assertEquals("The validation was expected to " + (pass ? "pass" : "fail"), pass, validation);
	}
	

	/**
	 * testDataCenterAbstract 
	 * testDataCenterFiles This test case will require the presence of "DataCenterSample" in the context workspace
	 * @throws Exception
	 */
	public void testDataCenterAbstract() throws Exception
	{
		validateModelUnits(
				new String[] {"/DataCenterSample/abstract"}, 
				"D://temp//testDataCenterFiles.output",
				"/expected-results/common/empty.output", true);
	}

	/**
	 * testDataCenterHardware
	 * @throws Exception
	 */
	public void testDataCenterHardware() throws Exception
	{
		validateModelUnits(
				new String[] {"/DataCenterSample/hardware"}, 
				"D://temp//testDataCenterHardware.output",
				"/expected-results/common/empty.output", true);
	}

	/**
	 * testDataCenterOS
	 * @throws Exception
	 */
	public void testDataCenterOS() throws Exception
	{
		validateModelUnits(
				new String[] {"/DataCenterSample/os"}, 
				"D://temp//testDataCenterOS.output",
				"/expected-results/common/empty.output", true);
	}

	/**
	 * testDataCenterSuperTypes
	 * @throws Exception
	 */
	public void testDataCenterSuperTypes() throws Exception
	{
		validateModelUnits(
				new String[] {"/DataCenterSample/samplesupertypes/"}, 
				"D://temp//testDataCenterSuperTypes.output",
				"/expected-results/common/empty.output", true);
	}

	/**
	 * testDataCenterSoftware
	 * @throws Exception
	 */
	public void testDataCenterSoftware() throws Exception
	{
		validateModelUnits(
				new String[] {"/DataCenterSample/software"}, 
				"D://temp//testDataCenterSoftware.output",
				"/expected-results/common/empty.output", true);
	}

	/**
	 * testDataCenterAll
	 * @throws Exception
	 */
	public void testDataCenterAll() throws Exception
	{
		validateModelUnits(
				new String[] {
						"/DataCenterSample/abstract",
						"/DataCenterSample/hardware",
						"/DataCenterSample/os",
						"/DataCenterSample/samplesupertypes",
						"/DataCenterSample/software"
						}, 
				"D://temp//testDataCenterAll.output",
				"/expected-results/common/empty.output", true);
	}
	
	
	/**
	 * testAcyclicInvalidCycles
	 * This test case will require the presence of "TestSMLSample"
	 * in the context workspace
	 * @throws Exception
	 */
	public void testAcyclicInvalidCycles() throws Exception
	{
		validateModelUnits(
				new String[] {"/TestSMLSample/acyclic/InValidCycles"}, 
				"D://temp//testAcyclicInvalidCycles.output",
				"/expected-results/other/acycliInvalidCycles.output", false);
	}

	/**
	 * testAcyclicInvalidDerivation
	 * @throws Exception
	 */
	public void testAcyclicInvalidDerivation() throws Exception
	{
		validateModelUnits(
				new String[] {"/TestSMLSample/acyclic/InValidDerivation"}, 
				"D://temp//testAcyclicInvalidDerivation.output",
				"/expected-results/other/acyclicInvalidDerivation.output", false);
	}

	/**
	 * testAcyclicValidCycles1
	 * @throws Exception
	 */
	public void testAcyclicValidCycles1() throws Exception
	{
		validateModelUnits(
				new String[] {"/TestSMLSample/acyclic/ValidCycles1"}, 
				"D://temp//testAcyclicValidCycles1.output",
				"/expected-results/common/empty.output", true);
	}

	/**
	 * testAcyclicValidCycles2
	 * @throws Exception
	 */
	public void testAcyclicValidCycles2() throws Exception
	{
		validateModelUnits(
				new String[] {"/TestSMLSample/acyclic/ValidCycles2"}, 
				"D://temp//testAcyclicValidCycles2.output",
				"/expected-results/common/empty.output", true);
	}

	/**
	 * testValidDerivation
	 * @throws Exception
	 */
	public void testValidDerivation() throws Exception
	{
		validateModelUnits(
				new String[] {"/TestSMLSample/acyclic/ValidDerivation"}, 
				"D://temp//testValidDerivation.output",
				"/expected-results/common/empty.output", true);
	}
}
