package org.eclipse.cosmos.rm.validation.internal.smlvalidators;

import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.NamespaceContext;
import javax.xml.parsers.ParserConfigurationException;

import junit.framework.Test;

import org.eclipse.cosmos.rm.validation.internal.common.FileOutput;
import org.eclipse.cosmos.rm.validation.internal.common.IValidationConstants;
import org.eclipse.cosmos.rm.validation.internal.common.IValidationOutput;
import org.eclipse.cosmos.rm.validation.internal.common.UtilityTester;
import org.eclipse.cosmos.rm.validation.internal.core.IValidator;
import org.eclipse.cosmos.rm.validation.internal.databuilders.AbstractDataBuilder;
import org.eclipse.cosmos.rm.validation.internal.databuilders.DataBuilderRegistry;
import org.eclipse.cosmos.rm.validation.internal.databuilders.DocumentDOMBuilder;
import org.eclipse.cosmos.rm.validation.internal.databuilders.IdentityConstraintDataBuilder;
import org.eclipse.cosmos.rm.validation.internal.databuilders.IdentityConstraintDataBuilder.IdentityConstraintDecl;
import org.eclipse.cosmos.rm.validation.internal.databuilders.IdentityConstraintDataBuilder.IdentityConstraintStructure;
import org.eclipse.hyades.test.common.junit.DefaultTestArbiter;
import org.eclipse.hyades.test.common.junit.HyadesTestCase;
import org.eclipse.hyades.test.common.junit.HyadesTestSuite;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Generated code for the test suite <b>TestIdentityConstraintValidator</b> located at
 * <i>/org.eclipse.cosmos.rm.validation.tests/junits/TestIdentityConstraintValidator.testsuite</i>.
 */
public class TestIdentityConstraintValidator extends HyadesTestCase
{
	/**
	 * Constructor for TestIdentityConstraintValidator.
	 * @param name
	 */
	public TestIdentityConstraintValidator(String name)
	{
		super(name);
	}

	/**
	 * Returns the JUnit test suite that implements the <b>TestIdentityConstraintValidator</b> definition.
	 */
	public static Test suite()
	{
		HyadesTestSuite testIdentityConstraintValidator = new HyadesTestSuite("TestIdentityConstraintValidator");
		testIdentityConstraintValidator.setArbiter(DefaultTestArbiter.INSTANCE).setId("E7509F3E43B7507FA7EA0A5092E911DB");
		testIdentityConstraintValidator.addTest(new TestIdentityConstraintValidator("testInvalidKeyDuplicate").setId("E7509F3E43B7507FAA9CA73092E911DB").setTestInvocationId("E7509F3E43B7507FD8F99670931A11DB"));
		testIdentityConstraintValidator.addTest(new TestIdentityConstraintValidator("testInvalidKeyMissing").setId("FA3DD97E7DA4834C5EF6F5E0947111DB").setTestInvocationId("FA3DD97E7DA4834C651CE790947111DB"));
		testIdentityConstraintValidator.addTest(new TestIdentityConstraintValidator("testInvalidUnique").setId("FA3DD97E7DA4834CF82ABDF0948A11DB").setTestInvocationId("FA3DD97E7DA4834C15365990948B11DB"));
		testIdentityConstraintValidator.addTest(new TestIdentityConstraintValidator("testValidKeyref").setId("CA88AFF1D7B860F561B19B10950B11DB").setTestInvocationId("CA88AFF1D7B860F56CB03DF0950B11DB"));
		testIdentityConstraintValidator.addTest(new TestIdentityConstraintValidator("testValidKeyUnique").setId("CA88AFF1D7B860F5DAED1190955511DB").setTestInvocationId("CA88AFF1D7B860F5E3BAF1C0955511DB"));
		testIdentityConstraintValidator.addTest(new TestIdentityConstraintValidator("testValidKeyUniqueInScopeOnly").setId("CA88AFF1D7B860F5848C136095EA11DB").setTestInvocationId("CA88AFF1D7B860F58EE74A5095EA11DB"));
		return testIdentityConstraintValidator;
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception
	{
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception
	{
	}

	
	private NamespaceContext createNamespaceContext(String[][] urlMapping)
	{
		final Map namespaceMap = new Hashtable();
		for (int i = 0; i < urlMapping.length; i++)
		{
			namespaceMap.put(urlMapping[i][0], urlMapping[i][1]);
		}
		
		return new NamespaceContext(){

			public String getNamespaceURI(String prefix)
			{
				String uri = (String)namespaceMap.get(prefix);			
				return uri == null ? IValidationConstants.EMPTY_STRING : uri;
			}

			public String getPrefix(String arg0)
			{
				/* Doesn't need to be implemented */
				return null;
			}

			public Iterator getPrefixes(String arg0)
			{
				/* Doesn't need to be implemented */
				return null;
			}};
	}

	private Map setupOutput(String path)
	{
		FileOutput fileOutput = new FileOutput();
		Map atts = new Hashtable();
		atts.put(IValidator.ATTRIBUTE_FILE_OUTPUT, path);		
		fileOutput.initialize(atts);
		Map validatorAtts = new Hashtable();
		validatorAtts.put(IValidator.ATTRIBUTE_OUTPUT, fileOutput);		
		return validatorAtts;
	}

	private void createIdentityConstraint(Object[][] declaredConstraints, Object[][] instancesTobechecked)
	{
		final IdentityConstraintStructure identityStruct = new IdentityConstraintStructure();
		Map declarations = new Hashtable();
		for (int i = 0; i < declaredConstraints.length; i++)
		{
			IdentityConstraintDecl[] constraints = (IdentityConstraintDecl[])declaredConstraints[i][2];
			List constraintList = new ArrayList();
			for (int j = 0; j < constraints.length; j++)
			{
				constraintList.add(constraints[j]);
			}
			Map localName = new Hashtable();
			localName.put(declaredConstraints[i][1], constraintList);
			declarations.put(declaredConstraints[i][0], localName);
		}
		
		identityStruct.setDeclarations(declarations);
		Map constrainedInstances = new Hashtable();
		Map orphanedInstances = new Hashtable();
		for (int i = 0; i < instancesTobechecked.length; i++)
		{
			List indices = new ArrayList();
			indices.add(instancesTobechecked[i][2]);
			if (instancesTobechecked[i][0] == null)
			{
				orphanedInstances.put(instancesTobechecked[i][1], indices);
			}
			else
			{
				constrainedInstances.put(instancesTobechecked[i][0], indices);
			}
		}
		identityStruct.setConstrainedInstances(constrainedInstances);
		identityStruct.setOrphanedConstrainedInstances(orphanedInstances);		
		
		DataBuilderRegistry.instance().registerDataStructureBuilder(IdentityConstraintDataBuilder.ID, new AbstractDataBuilder(){

			public Object getDataStructure()
			{
				return identityStruct;
			}});
	}

	private void createDOMStructure(String universityFile, String[] students, String[] courses) throws ParserConfigurationException, SAXException, IOException
	{
		Document document = UtilityTester.domParseDocument(this.getClass().getClassLoader().getResourceAsStream(universityFile));
		final Map domDocuments = new Hashtable();
		List orphanList = new ArrayList();
		orphanList.add(document.getChildNodes().item(0));
		domDocuments.put(DocumentDOMBuilder.NO_ALIAS, orphanList);

		addDocuments (domDocuments, "Student", students);
		addDocuments (domDocuments, "Course", courses);
		
		DataBuilderRegistry.instance().registerDataStructureBuilder(DocumentDOMBuilder.ID, new AbstractDataBuilder(){

			public Object getDataStructure()
			{
				return domDocuments;
			}
		});
	}

	private void addDocuments(Map domDocuments, String base, String[] rawFragments) throws ParserConfigurationException, SAXException, IOException
	{
		if (rawFragments == null)
			return;
		
		Document document;
		for (int i = 0; i < rawFragments.length; i++)
		{
	        document = UtilityTester.domParseDocument(new InputSource(new StringReader(rawFragments[i])));
	        domDocuments.put(base + (i+1), document.getChildNodes().item(0));
	    }
	}
	
	
	private IdentityConstraintDecl createConstraint (String name, String reference, byte type, String selector, String[] field)
	{
		IdentityConstraintDecl constraint = new IdentityConstraintDecl(name);
		constraint.setType(type);
		constraint.setSelector(selector);
		constraint.setReference(reference);
		for (int i = 0; i < field.length; i++)
		{
			constraint.addField(field[i]);
		}
		
		constraint.setNamespaceContext(createNamespaceContext(new String[][]{
				new String[]{"tns", "urn:university"},
				new String[]{"xs", "http://www.w3.org/2001/XMLSchema"},
				new String[]{"sml", "http://schemas.serviceml.org/sml/2006/10"},
				new String[]{"smlfn", "http://schemas.serviceml.org/sml/function/2006/07"}
		}));
		
		return constraint;
	}
	
	
	private void runValidation(String fileName, String[][] reftypes, IdentityConstraintDecl[] constraint, 
			String universityFile, String[] students, String[] courses) throws Exception
	{
		Map validationAttributes = null;
		String outputFile = null;
		IValidationOutput output = null;
		if (fileName != null)
		{
			outputFile = "D://temp//" + fileName;		
			validationAttributes = setupOutput(outputFile);
			output = (IValidationOutput)validationAttributes.get(IValidator.ATTRIBUTE_OUTPUT);
		}
		
		try
		{		
			IdentityConstraintValidator identityValidator = new IdentityConstraintValidator();
			identityValidator.initialize(validationAttributes == null ? new Hashtable() : validationAttributes);
			
			UtilityTester.registerReftypeElements(reftypes);
			UtilityTester.registerIdentityStructure(null, null, "http://interop.serviceml.org");
			createDOMStructure(universityFile, students, courses);			
			createIdentityConstraint(
					new Object[][]{
						new Object[]{"urn:university", "University", constraint}},
					new Object[][] {
						new Object[]{null, "0", new int[]{0}}});
			boolean validation = identityValidator.validate();
			
			if (fileName == null)
			{
				assertTrue("The identity constraint validation was expected to pass", validation);
			}
			else
			{
				assertFalse("The identity constraint validation was expected to fail", validation);
				UtilityTester.compareFiles("expected-results/identity/" + fileName, outputFile);
			}
		}
		finally
		{
			if (output != null)
				output.close();
		}
	}
	
	private String[] createStudents(int[] ids, String[] ssn, String[] names)
	{
		List students = new ArrayList(names.length);
		for (int i = 0; i < names.length; i++)
		{
	        String student = 	
	        	"<Student xmlns=\"urn:university\"" + (ssn[i] == null ? ">" : " SSN=\"" + ssn[i] + "\">") +
	        		"<ID>" + ids[i] + "</ID>" +
	        		"<Name>" + names[i] + "</Name>" +
				"</Student>";
	        students.add(student);
		}

		return (String[])students.toArray(new String[students.size()]);
	}
	
	private String[] createCourses (String[][] enrolledStudents)
	{
		if (enrolledStudents == null)
			return new String[0];
		
		List courses = new ArrayList(enrolledStudents.length);
		for (int i = 0; i < enrolledStudents.length; i++)
		{
	        String course = 	
	        	"<Course xmlns=\"urn:university\" xmlns:sml=\"http://schemas.serviceml.org/sml/2006/10\">"+
	        		"<Name>Course" + i + "</Name>";
	        for (int j = 0; j < enrolledStudents[i].length; j++)
			{
	        	course += "<EnrolledStudent sml:ref=\"true\">" +
	            			"<sml:uri>" + enrolledStudents[i][j] + "</sml:uri>" +
	            		  "</EnrolledStudent>";
			}
	        
	        course += "</Course>";
	        courses.add(course);
		}
		
		return (String[])courses.toArray(new String[courses.size()]);
	}

	/**
	 * testInvalidKeyDuplicate
	 * @throws Exception
	 */
	public void testInvalidKeyDuplicate() throws Exception
	{			
		runValidation(
				"testInvalidKeyDuplicate.output", 
				new String[][] {new String[]{"urn:university", "EnrolledStudent"}},
				new IdentityConstraintDecl[]{
								 createConstraint("StudentIDisKey", null, IdentityConstraintDecl.KEY_TYPE, 
								 "smlfn:deref(tns:EnrolledStudents/tns:EnrolledStudent)", 
								 new String[]{"tns:ID"})},
				"test-resources/identity/University.xml",
				createStudents(  new int[] {1000, 1001, 1002, 1000},
								 new String[] {null, null, null, null},
								 new String[] {"John", "John", "John", "Ray"}),
				
				createCourses(null));
	}
	
	
	/**
	 * testInvalidKeyMissing
	 * @throws Exception
	 */
	public void testInvalidKeyMissing() throws Exception
	{
		runValidation(
				"testInvalidKeyMissing.output", 
				new String[][] {new String[]{"urn:university", "EnrolledStudent"}},
				new IdentityConstraintDecl[]{
								 createConstraint("StudentSSNisKey", null, IdentityConstraintDecl.KEY_TYPE, 
								 "smlfn:deref(tns:EnrolledStudents/tns:EnrolledStudent)", 
								 new String[]{"tns:@SSN"})},
				"test-resources/identity/University.xml",
				createStudents(
								 new int[] {1000, 1001, 1002, 1000},
								 new String[] {"123-22-4444", "333-22-4444", null, "111-22-3333"},
								 new String[] {"John", "John", "John", "Ray"}),
				createCourses(null));
	}
	

	/**
	 * testInvalidUnique
	 * @throws Exception
	 */
	public void testInvalidUnique() throws Exception
	{
		runValidation(
				"testInvalidUnique.output", 
				new String[][] {new String[]{"urn:university", "EnrolledStudent"}},
				new IdentityConstraintDecl[]{
								 createConstraint("StudentSSNisUnique", null, IdentityConstraintDecl.UNIQUE_TYPE, 
								 "smlfn:deref(tns:EnrolledStudents/tns:EnrolledStudent)", 
								 new String[]{"tns:Name", "tns:@SSN"})},
				"test-resources/identity/University.xml",
				createStudents (
								 new int[] {1000, 1001, 1002, 1004}, 
								 new String[] {"123-22-4444", "333-22-4444", "123-22-4444", "111-22-3333"},
								 new String[] {"John", "Jane", "John", "Ray"}),
				createCourses(null));
	}

	/**
	 * testValidKeyref
	 * @throws Exception
	 */
	public void testValidKeyref() throws Exception
	{
		runValidation(
				null, 
				new String[][] { new String[]{"urn:university", "EnrolledStudent"},
								 new String[]{"urn:university", "EnrolledCourse"},
								 new String[]{"urn:university", "OfferedCourse"}},
				new IdentityConstraintDecl[]{				 
								 createConstraint("StudentIDisKey", null, IdentityConstraintDecl.KEY_TYPE, 								
								 "smlfn:deref(tns:EnrolledStudent)", 
								 new String[]{"tns:ID"}),
								 
								 createConstraint("CourseStudents", "StudentIDisKey",
										 IdentityConstraintDecl.KEY_REF_TYPE, 
										 "smlfn:deref(smlfn:deref(tns:OfferedCourse)/tns:EnrolledStudent)", 
										 new String[]{"tns:ID"})
								 },								 
				"test-resources/identity/University2.xml",				
				createStudents(  new int[] {1000, 1001, 1002, 1004},
								 new String[] {null, null, null, null},
								 new String[] {"John", "John", "John", "Ray"}),
				
				createCourses(new String[][]{
								 new String[] {"Student1", "Student2"},
								 new String[] {"Student3", "Student2", "Student4"}
				}));
	}

	/**
	 * testValidKeyUnique
	 * @throws Exception
	 */
	public void testValidKeyUnique() throws Exception
	{
		runValidation(
				null, 
				new String[][] { new String[]{"urn:university", "EnrolledStudent"}},
				new IdentityConstraintDecl[]{				 
								 createConstraint("StudentSSNisUnique", null, IdentityConstraintDecl.UNIQUE_TYPE, 								
										 "smlfn:deref(tns:EnrolledStudents/tns:EnrolledStudent)", 
										 new String[]{"tns:Name", "tns:@SSN"}),
								 
								 createConstraint("StudentIDisKey", null, IdentityConstraintDecl.KEY_TYPE, 
										 "smlfn:deref(tns:EnrolledStudents/tns:EnrolledStudent)", 
										 new String[]{"tns:ID"})
								 },								 
				"test-resources/identity/University.xml",				
				createStudents(  new int[] {1000, 1001, 1002, 1004},
								 new String[] {"123-22-4444", "333-22-4444", null, "111-22-3333"},
								 new String[] {"John", "John", "John", "Ray"}), 
				null);
	}

	/**
	 * testValidKeyUniqueInScopeOnly
	 * @throws Exception
	 */
	public void testValidKeyUniqueInScopeOnly() throws Exception
	{
		runValidation(
				null, 
				new String[][] { new String[]{"urn:university", "EnrolledStudent"}},
				new IdentityConstraintDecl[]{				 
								 createConstraint("StudentSSNisUnique", null, IdentityConstraintDecl.UNIQUE_TYPE, 								
										 "smlfn:deref(tns:EnrolledStudents/tns:EnrolledStudent)", 
										 new String[]{"tns:Name", "tns:@SSN"}),
								 
								 createConstraint("StudentIDisKey", null, IdentityConstraintDecl.KEY_TYPE, 
										 "smlfn:deref(tns:EnrolledStudents/tns:EnrolledStudent)", 
										 new String[]{"tns:ID"})
								 },								 
				"test-resources/identity/University.xml",				
				createStudents(  new int[] {1000, 1001, 1002, 1004, 1004, 1000},
								 new String[] {"123-22-4444", "333-22-4444", null, "111-22-3333", "111-22-3333", "123-22-4444"},
								 new String[] {"John", "John", "John", "Ray", "Ray", "John"}), 
				null); 
	}
}
