/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.databuilders;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.cosmos.rm.validation.internal.SMLActivator;
import org.eclipse.cosmos.rm.validation.internal.artifacts.ElementTypeMap;
import org.eclipse.cosmos.rm.validation.internal.artifacts.TargetSchemaNode;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.eclipse.cosmos.rm.validation.internal.common.SMLCommonUtil;
import org.eclipse.cosmos.rm.validation.internal.util.ParserHelper;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

public class TargetSchemaBuilder extends AbstractDataBuilder implements
		IDataBuilder {

	public static final String ID = SMLActivator.PLUGIN_ID + ".TargetSchemaBuilder";
	
	// TODO: (ALI):  Sheldon, this map is not namespace aware.  It seems that you're storing local element names
	// and their associated target constraint. 
	protected Map dataStructure;
    protected boolean uriElementFound = false; 	
	
	public TargetSchemaBuilder(){		
		dataStructure = new HashMap();
	}

	public Object getDataStructure() {
		return dataStructure;
	}

	public void startSchema(String uri) {
	}
	
	public void endSchema(String uri) 
	{
		Map substitionMap = (Map)SMLCommonUtil.retrieveDataStructure(SubstitutionBuilder.ID);
		
		//validate if target elements is defined on a derived type
		IDataBuilder refTypeElement = DataBuilderRegistry.instance().getDataStructureBuilder(RefTypeElementBuilder.ID);
		IDataBuilder elementTypeMapBuilder  = DataBuilderRegistry.instance().getDataStructureBuilder(ElementTypeMapDataBuilder.ID);
		ElementTypeMap elementTypeMapBuilderStructure = (ElementTypeMap)elementTypeMapBuilder.getDataStructure();
		IDataBuilder typeInheritance = DataBuilderRegistry.instance().getDataStructureBuilder(TypeInheritanceDataBuilderImpl.ID);
		Map refTypeElems = (Map)refTypeElement.getDataStructure();
		Map inheritance = (Map)typeInheritance.getDataStructure();
		
		Iterator iter = dataStructure.keySet().iterator();
		while (iter.hasNext()){
			Object elementName = iter.next();
			if (refTypeElems.containsKey(elementName))
				continue;
			//check inheritance tree
			Object targetRefTypeNS = elementTypeMapBuilderStructure.getType(uri, (String)elementName);
			String targetRefType = ParserHelper.removeNameSpace((String)targetRefTypeNS);
			Object parent = inheritance.get(targetRefType);
			boolean valid  = false;
			while (parent != null){
				if (refTypeElems.get(parent) != null){
					valid = true;
					break;
				}
				parent = inheritance.get(parent);				
			}
			if (!valid){				
				appendToErrorMessage("Target Element does not define sml:refType");
				return;
			}
		}
		
		// TODO: (ALI): Sheldon this is not going to work if the referenced substitutedGroup element is loaded after the element containing
		// the substitutedGroup attribute.
		//need to make sure that substitionElements are also considered
		iter = substitionMap.keySet().iterator();
		while (iter.hasNext())
		{
			Object namespaceURI = iter.next();
			String elementNameNonNS = ParserHelper.removeNameSpace((String)namespaceURI);
			TargetSchemaNode targetSchemaNode = (TargetSchemaNode)dataStructure.get(elementNameNonNS);
			
			
			if (targetSchemaNode != null)
			{
				Object substitionName = substitionMap.get(namespaceURI);
				TargetSchemaNode subTargetSchemaNode = (TargetSchemaNode)dataStructure.get(substitionName);
				targetSchemaNode.setSubstitutionGroup(subTargetSchemaNode);				
			}
			else
			{
				Map substitutionMap = (Map)substitionMap.get(namespaceURI);
				for (Iterator subtitutions = substitutionMap.keySet().iterator(); subtitutions.hasNext();)
				{
					String elementName = (String) subtitutions.next();
					String substitutedElement = (String)substitutionMap.get(elementName);
					substitutedElement = substitutedElement == null ? null : ParserHelper.removeNameSpace(substitutedElement);
					TargetSchemaNode subTargetSchemaNode = (TargetSchemaNode)dataStructure.get(substitutedElement);	
					if (subTargetSchemaNode != null)
					{
						dataStructure.put(elementNameNonNS, subTargetSchemaNode);
					}					
				}
											
			}
		}
		
	}
	

	public void startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
		if (ISMLConstants.SCHEMA_ELEMENT.equals(localName)){
			startSchema(uri);
		}
		else if (ISMLConstants.ELEMENT_ELEMENT.endsWith(localName)){
			String elementName = attributes.getValue(ISMLConstants.NAME_ATTRIBUTE);
			String targetElement = attributes.getValue(ISMLConstants.SML_URI, ISMLConstants.TARGETELEMENT_ATTRIBUTE);
			String targetType = attributes.getValue(ISMLConstants.SML_URI, ISMLConstants.TARGETTYPE_ATTRIBUTE);
			String targetRequired = attributes.getValue(ISMLConstants.SML_URI, ISMLConstants.TARGETREQUIRED_ATTRIBUTE);
			if ((targetElement != null) || (targetType != null) || (targetRequired != null)){ 
				TargetSchemaNode schemaNode = new TargetSchemaNode(elementName, locator == null ? -1 : locator.getLineNumber(), locator == null ? -1 : locator.getColumnNumber());;
				schemaNode.setTargetElement(targetElement);
				schemaNode.setTargetType(targetType);
				schemaNode.setRequired(targetRequired);
				dataStructure.put(elementName, schemaNode);
			}
		
		}					
	}

	public void endElement(String uri, String localName, String qName) throws SAXException {
		if (ISMLConstants.SCHEMA_ELEMENT.equals(localName)){
			endSchema(uri);
		}
	}

	public byte getPhase() {
		return ISMLConstants.GENICS_PHASE;
	}	

}
